@@ -16,6 +16,7 @@
 #' @param addName When TRUE the variable name is added to the level names, except for variables with most levels.
 #' @param sep A character string to separate when addName apply.
 #' @param removeZeros When TRUE, rows with zero count will be removed from the data.
+#' @param dimList See \code{\link{ProtectTable}}.
 #' @param groupVarInd Possible manual specification if list defining the hierarchical variable groups
 #' @param ind1  Coding of table 1 as indices referring to elements of groupVarInd
 #' @param ind2  Coding of table 2 as indices referring to elements of groupVarInd
@@ -54,19 +55,21 @@
 #'         \code{\link{FindDimLists}}, \code{\link{FindCommonCells}}
 #'
 #' @examples
-#' z  <- EasyData("sosialFiktiv")
-#'
-#' z8 <- z[z$fylke<=10 & z$kostragr==300,] # 8 regions
-#' a  <- ProtectTable1(z8,c(1,2,4,5),7)
-#' as.data.frame(getInfo(a[[1]][[1]],type="finalData")) # The table (not two linked)
-#'
-#' z36 <- z[z$fylke>=11 & z$fylke<=14 & z$kostragr<=500,] # 36 regions
-#' b <- ProtectTable1(z36,1:6,7)
-#' as.data.frame(getInfo(b[[1]][[1]],type="finalData")) # First table
-#' as.data.frame(getInfo(b[[2]][[1]],type="finalData")) # Second table
+#' \dontrun{
+#' z2 <- EasyData("z2")
+#' a <- ProtectTable1(z2, c(1, 3, 4), 5)
+#' head(as.data.frame(getInfo(a[[1]][[1]], type = "finalData"))  # The table (not two linked))
+#' 
+#' z3 <- EasyData("z3")
+#' b <- ProtectTable1(z3, 1:6, 7)
+#' head(as.data.frame(getInfo(b[[1]][[1]], type = "finalData")))  # First table
+#' head(as.data.frame(getInfo(b[[2]][[1]], type = "finalData")))  # Second table
+#' }
 ProtectTable1 <- function(data, dimVarInd = 1:NCOL(data), freqVarInd = NULL, protectZeros = TRUE, 
                           maxN = 3, method = "SIMPLEHEURISTIC", findLinked = TRUE, total = "Total", addName = FALSE, 
-                          sep = ".", removeZeros = FALSE, groupVarInd = NULL, ind1 = NULL, ind2 = NULL, 
+                          sep = ".", removeZeros = FALSE, 
+                          dimList = NULL,
+                          groupVarInd = NULL, ind1 = NULL, ind2 = NULL, 
                           dimDataReturn = FALSE, 
                           IncProgress = IncDefault,
                           ...) {
@@ -112,11 +115,19 @@ ProtectTable1 <- function(data, dimVarInd = 1:NCOL(data), freqVarInd = NULL, pro
   dimLists <- FindDimLists(data[, dimVarInd, drop = FALSE], groupVarInd = groupVarInd, 
                            addName = addName, sep = sep, total = total, xReturn = dimDataReturn)
   
+
   if (dimDataReturn) {
     dimData <- dimLists$x
     dimLists <- dimLists$dimLists
   } else dimData <- NULL
   
+  
+  if(!is.null(dimList)){
+    dimLists <- ReplaceDimList(dimLists, dimList, total = total)
+  }
+  
+  
+  
   dimList1 <- dimLists[ind1]
   
   if(makeMicro){ 
